#ifndef __cellspace__h_
#define __cellspace__h_
#include "cell.h"
/**
The cellspace is a network executive containing a grid of cells.  The cell space
wraps at the edges.
*/
class cellspace: public netExec 
{
	public:		
		// constructs a cellspace with the indicated width and height.
		cellspace(int width, int height, double D, double dx, double dy, double k);
		// The compute neighhbors method is called by the simulator when messages
		// appear on the output port of a cell.  Returns the cells that form the
		// event generating cell's Moore neighborhood (minus the cell itself).
		void route(const PortValue& pv, devs* src, adevs_bag<EventReceiver>& r);
		// Get all the cells in the space
		void getComponents(adevs_set<devs*>& c);
		void report(ofstream& stream);
		void init() { passivate(); }
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x){}
		void delta_int(){}
		void delta_conf(const adevs_bag<PortValue>& x){}
		void output_func(adevs_bag<PortValue>& y){}
		void gc_output(adevs_bag<PortValue>& g){}
		void gc_models(adevs_set<devs*>& models){}
		~cellspace();
		void set_u(int x, int y, double value) { cell::set_u(x,y,value); }
	private:	
		cell** space;
};

class world: public devsn
{
	public:
		world(int width, int height, double D, double dx, double dy, double k):
		devsn(),
		nx(width,height,D,dx,dy,k){}
		netExec* getNetExec(){ return &nx; }
		void report(ofstream& log)
		{
			nx.report(log);
		}
		void set_u(int x, int y, double value) { nx.set_u(x,y,value); }
	private:
		cellspace nx;
};

#endif
