#include "cellspace.h"
#include "configuration.h"
#include <iostream>
#include <fstream>
#include <stdio.h>

class world_sim: public devssim
{
	public:
		world_sim(world* model, double sample_interval):
		devssim(model),model(model),first(true),
		sample_interval(sample_interval),last_sample(0.0){}
		bool halt()
		{
			if (timeLast() - last_sample >= sample_interval || first)
			{
				first = false;
				cout << "Sampling @ " << timeLast() << endl;
				char file_name[100];
				sprintf(file_name,"log-%f.sample",timeLast());
				ofstream log(file_name);
				model->report(log);
				log.close();
				last_sample = timeLast();
			}
			return devssim::halt();
		}
	private:
		world* model;
		bool first;
		double sample_interval, last_sample;
};

int main(int argc, char** argv)
{
	if (argc < 2)
	{
		std::cerr << argv[0] << " <configuration file>" << std::endl;
		std::cerr << "No input file" << std::endl;
		return 0;
	}
	configuration config(argv[1]);
	setAdevsTimeGranule(config.get_granule());
	world model(config.get_width(),config.get_height(),
		config.get_D(),config.get_dx(),config.get_dy(),config.get_k());

	int x, y;
	double value;

	config.start_u_data();
	while (!config.end_of_data())
	{
		config.next(x,y,value);
		model.set_u(x,y,value);
	}
	cout << "Running..." << endl;
	world_sim sim(&model,config.get_tsample());	
	sim.run(config.get_tend());
	return 0;
}
