#include <adevs_hashtable.h>
#include <assert.h>
#include <perf_timer.h>
#include <iostream>
#define ARRAY_SIZE 3000

void test1()
{
	adevs_hashtable<int,int> t;
	t[1] = 1;
	t[2] = 2;
	t.remove(1);
	assert(t.getSize() == 1);
	t.remove(2);
	assert(t.getSize() == 0);
}

void test2()
{
	adevs_hashtable<int,int> t;
	t[1] = 1;
	t[2] = 2;
	t[1] = 2;
	assert(t[1] == 2);
	t.remove(1);
	assert(t.getSize() == 1);
	t.remove(2);
	assert(t.getSize() == 0);
}

void test3()
{
	adevs_hashtable<int,int> t;
	for (int i = 0; i < 200; i++)
	{
		t[i] = i;
		assert(t.contains(i));
	}
	cout << t.getRehashCount() << endl;
	assert(t.getSize() == 200);
	for (int i = 0; i < 200; i++)
	{
		t.remove(i);
		assert(!t.contains(i));
	}
	cout << t.getRehashCount() << endl;
	assert(t.getSize() == 0);
	for (int i = 0; i < 200; i++)
	{
		t[i] = i;
		assert(t.contains(i));
	}
	assert(t.getSize() == 200);
	t.clear();
	for (int i = 0; i < 200; i++)
	{
		assert(!t.contains(i));
	}
}

void test4() 
{
	int keys [ARRAY_SIZE];
	int values [ARRAY_SIZE];

	for (unsigned i = 0; i < ARRAY_SIZE; i++) 
	{
		keys[i] = i;
		values[i] = i+ARRAY_SIZE;
	}

	tic();
	adevs_hashtable<int,int> t;

	for (int i = 0; i < ARRAY_SIZE; i++) 
	{
		assert (t.getSize() == i);
		t.insert(keys[i], values[i]);
		assert(t.find(keys[i]) == values[i]);
		assert(t[keys[i]] == values[i]);
		t[keys[i]] = values[i] = i;
		assert(t[keys[i]] == values[i]);
		assert(t.contains(keys[i]));
	}
	cout << toc() << endl;

	assert(t.getSize() == ARRAY_SIZE);
	tic();
	adevs_hashtable<int,int> at(t);
	adevs_hashtable<int,int> bt = t;
	cout << toc() << endl;
	assert(at.getSize() == t.getSize());
	assert(bt.getSize() == t.getSize());
	tic();
	t.clear();
	cout << toc() << endl;
	assert(t.getSize() == 0);
	assert(t.contains(1) == false);
	tic();
	for (int i = 0; i < ARRAY_SIZE; i++) 
	{
		assert(at[keys[i]] == values[i]);
		assert(bt[keys[i]] == values[i]);
   }
	cout << toc() << endl;
	assert(at.getSize() == ARRAY_SIZE);
	assert(bt.getSize() == ARRAY_SIZE);
    
	tic();
	for (int i = 0; i < ARRAY_SIZE; i++) 
	{
		int size = at.getSize();
		at.remove(keys[i]);
		assert (at.getSize() == size-1);
		bt.remove(keys[i]);
		assert(at.contains(keys[i]) == false);
		assert(bt.contains(keys[i]) == false);
	}
	cout << toc() << endl;
	
	for (int i = 0; i < ARRAY_SIZE; i++) 
	{
		assert (t.getSize() == i);
		t.insert(keys[i], values[i]);
		assert(t.find(keys[i]) == values[i]);
		assert(keys[i] == values[i]);
		t[keys[i]] = values[i] = i;
		assert(keys[i] == values[i]);
		assert(t.contains(keys[i]));
	}
	assert(t.getSize() == ARRAY_SIZE);
}

int main()
{
	test1();
	test2();
	test3();
	test4();
	return 0;
}
