#include <iostream>
#include <adevs.h>
#include <assert.h>

class genr: public atomic
{
	public:
		static const port_t out;

		genr(stime_t period, int ticks):
		atomic(),period(period),ticks(ticks),count(0) {}
		void init()
		{
			count = 0;
			hold(period);
			assert(timeCurrent() == ZERO);
			assert(timeLast() == ZERO);
		}
		void delta_int()
		{
			assert(timeCurrent() == timeLast()+ta());
			assert(elapsed() == ta());
			count++;
			assert(count <= ticks);
			if (count == ticks) passivate();
			else hold(period);
		}
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x)
		{
			assert(false);
		}
		void delta_conf(const adevs_bag<PortValue>& x)
		{
			assert(false);
		}
		void output_func(adevs_bag<PortValue>& y)
		{
			output(out,new object(),y);
		}
		void gc_output(adevs_bag<PortValue>& g)
		{
			for (int i = 0; i < g.getSize(); i++) 
				delete g[i].value;
		}
		~genr(){}
		int getTickCount() { return count; }
	private:
		stime_t period;
		int ticks;
		int count;
};

const port_t genr::out(1);

void test1()
{
	genr* g = new genr(10.0,10);
	xdevssim sim(g);
	sim.initialize();
	while(sim.nextTN() < INFINITY)
	{
		sim.deltFunc(sim.nextTN());
	}
	sim.finalize();
	assert(g->getTickCount() == 10);
	delete g;
}

void test2()
{
	genr* g = new genr(-1.0,10);
	try
	{
		xdevssim sim(g);
		sim.initialize();
		assert(false);
	}
	catch(adevs_exception& err)
	{
		std::cerr<<err.getErrorMsg()<<std::endl;
	}
	delete g;
}

void test3()
{
	genr* g = new genr(1.0,10);
	xdevssim sim(g);
	sim.initialize();
	while (sim.nextTN() < INFINITY)
	{
		sim.computeInputOutput();
		sim.computeInputOutput();
		sim.deltFunc(sim.nextTN());
	}
	sim.finalize();
	assert(g->getTickCount() == 10);
	delete g;
}

int main()
{
	test1();
	test2();
	test3();
	return 0;
}
